local _version = '0.1.2'
local socket = require("socket");

local nextBuffer = 1;
local connected = false;
local shouldRetryConnection = false;
local authed = false;
--lsOneBuffer = {};
--lsTwoBuffer = {};
local otherbuffer = {};
--lsOne = {};
--lsTwo = {};
local echos = {};

local linkShells    = {}
linkShells[1] = {}
linkShells[2] = {}
local lsBuffers     = {}
lsBuffers[1]  = {}
lsBuffers[2]  = {}
local tcp = assert(socket.tcp());
local framework = nil

function hook (params) 
    framework = params
    sendNotice('LinkCloud Version: ' .. _version .. ' loaded.')
    if framework.settings.AutoConnect then
        ConnectToDiscordBot();
    else
        sendNotice('AutoConnect is disabled.  Use //lc connect to start streaming.')
    end;
	SendBuffer()
end

-- remove send packet from buffer (called after server confirms receipt)
function removePacketFromBuffers(packetId)
	for k, v in pairs(otherbuffer) do
		if (framework.json.decode(v)).packetId == packetId then
			table.remove(otherbuffer, k);
		end;
	end;
	for k, v in pairs(lsBuffers[2]) do
		if (framework.json.decode(v)).packetId == packetId then
			table.remove(lsBuffers[2], k);
		end;
	end;
	for k, v in pairs(lsBuffers[1]) do
		if (framework.json.decode(v)).packetId == packetId then
			table.remove(lsBuffers[1], k);
		end;
	end;
end;

-- Ticks every 250ms sends the next buffered item to the server
function SendBuffer()
	if authed then
		if tcp then
			nextMessage = nil;
			if nextBuffer == 1 and linkShells[1].name ~= nil then
				nextMessage = lsBuffers[1][1];
			elseif nextBuffer == 2 then
				nextMessage = otherbuffer[1];
			elseif nextBuffer == 3 and linkShells[2].name ~= nil then
				nextMessage = lsBuffers[2][1];
			end;
			if nextMessage and connected then
				sendToBot(nextMessage);
			end;
		end;
		nextBuffer = nextBuffer + 1;
		if nextBuffer > 3 then
			nextBuffer = 1;
		end;		
	end;
	coroutine.schedule(SendBuffer, 0.25);
end;

--Notify the user the connection has been lost.
--If auto reconnect is enabled, retry the connection in 30 seconds
function notifiyConnectionLost()
	connected = false;
	authed = false;
	tcp:close();
	if framework.settings.AutoReconnect then
		sendError("You have lost connection to the LinkCloud server.  The connection attempt will be retried shortly.");
		coroutine.schedule(ConnectToDiscordBot, 30);
	else
		sendError("You have lost connection to the LinkCloud server.  Use //lc connect to retry the connection.");
	end;
end;

--Parse /lsmes message
function getLSFromLsmes(str)
	lsData = {};
	lsData.shellNumber = 0;
	slugs = split(str, " ");
	if string.match(slugs[1], "(%d:)") then
		sendError("timestamp addon must be disabled.");
		notifiyConnectionLost();
	elseif slugs[1] == "[1]<" then
		lsData.shellNumber = 1;
		lsData.name = string.sub(slugs[2], 0, slugs[2]:len() - 1);
	elseif slugs[1] == "[2]<" then
		lsData.shellNumber = 2;
		lsData.name = string.sub(slugs[2], 0, slugs[2]:len() - 1);
	end;
	return lsData;
end;
--javascript like split function (thanks google)
function split(s, sep)
	local fields = {};
	local sep = sep or " ";
	local pattern = string.format("([^%s]+)", sep);
	string.gsub(s, pattern, function(c)
		fields[(#fields) + 1] = c;
	end);
	return fields;
end;
--uuidv4 style random generator
function uuid()
	local template = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx";
	return string.gsub(template, "[xy]", function(c)
		local v = c == "x" and math.random(0, 15) or math.random(8, 11);
		return string.format("%x", v);
	end);
end;
--Put all the packet peices together into one packet, convert to json
function buildPacket(type, payload)
	packet = {};
	packet.metaData = metaData();
	packet.type = type;
	packet.payload = payload;
	packet.packetId = uuid();
	return framework.json.encode(packet) .. "\n";
end;
--sets the authkey
function buildConnectionPacket()
	packet = {};
	packet.authId = framework.settings.AuthKey;
	return packet;
end;
--builds metadata to send along with the packet


function metaData()
	info = framework.getServerData();
	data = {};
    playerData = framework.getPlayerData()
	data.character = playerData and playerData.name;
	data.client_id = playerData and playerData.id;
	data.server = info.server;
	data.gameTime = info.time;
	data.clientTime = os.time();
	data.platform = framework.settings.Platform;
	return data;
end;
--builds a packet containing ls message data
function buildMessagePayload(message, ls)
	slugs = split(message, "> ");
	slugs = split(slugs[1], " <");
	nameparsed = slugs[2];
	message = message:gsub("%" .. slugs[1] .. "<" .. nameparsed .. "> ", "");
	payload = {};
	payload.name = nameparsed:gsub("[\n\r]", "");
	payload.message = message:gsub("[\n\r]", "");
	payload.linkshellname = ls:gsub("[\n\r]", "");
	return payload;
end;
--builds a packet containing shout data
function buildShoutPayload(msg)
	slugs = split(msg, ":");
	nameArea = slugs[1];
	table.remove(slugs, 1);
	nameAreaSlugs = split(nameArea, "[");
	message = table.concat(slugs, ":");
	name = nameAreaSlugs[1];
	area = nameAreaSlugs[2]:gsub("]", "");
	payload = {};
	payload.name = name;
	payload.area = area;
	payload.message = string.gsub(message, "^%s*(.-)%s*$", "%1");
	return payload;
end;
--build the "other" payload (testing only)
function buildOtherPayload(msg, mode, modemod, type)
	payload = {};
	payload.message = msg;
	payload.type = type;
	payload.mode = mode;
	payload.modemod = modemod;
	return payload;
end;

--force the game to display the lsmes for both linkshells (used to determine which ls is in which slot)
function getLSMes()
    framework.sendInputCommand("/lsmes")
	coroutine.sleep(2);
	framework.sendInputCommand("/ls2mes");
	
end;

--restart the receiver coroutine
function setupReceiver()
	RECEIVER_ROUTINE = coroutine.schedule(listen, 0.25);
end;

--closes the tcp connection if its open, resets connection and auth bits, recreate the tcp connection, perform auth challenge if connection is successful
function ConnectToDiscordBot()
	tcp:close();
	connected = false;
	authed = false;
	tcp = assert(socket.tcp());
	tcp:connect(framework.settings.HostAddress, framework.settings.HostPort);
	tcp:setoption("keepalive", true);
	tcp:settimeout(framework.settings.TCPTimeout);
	connectionpacket = tcp:receive("*l");
	if connectionpacket == "CHALLENGE" then
		connected = true;
		sendToBot(buildPacket("HANDSHAKE", buildConnectionPacket()));
		setupReceiver();
	elseif framework.settings.AutoReconnect then
		sendError("Unable to connect.")
		sendError("The discord bot is either offline or your connection information is incorrect.")
		sendError("Use //lc autoreconnect to stop reconnection attempts.");
		RECONNECT_ROUTINE = coroutine.schedule(ConnectToDiscordBot, 30);
	else
		sendError("Unable to connect.  The discord bot is either offline or your connection information is incorrect.");
	end;
end;
--wrapper for tcp:send
function sendToBot(payload)
	tcp:send(payload);
end;
--called on a coroutine on an interval when connected to retreive and process new socket data
function handleResponse()
	tcp:settimeout(0);
	response = tcp:receive("*l");
	packet = response and framework.json.decode(response);
	if packet then
		heartbeatReply = true;
		if packet.type == "HANDSHAKE" then
			if packet.payload == "ACCEPTED" then
				authed = true;
				coroutine.schedule(heartbeat, 1);
				sendNotice("Streaming to discord enabled.");
				getLSMes();
				SendBuffer();
			else
				sendError("LinkCloud handshake failed.  Please check your API key and try again.");
			end;
		elseif packet.type == "SYSTEM_MESSAGE" then
			if packet.payload.isError then
				sendError(packet.payload.message);
			else
				sendNotice(packet.payload.message);
			end;
		elseif packet.type == "LS_ECHO" then
			if packet.payload.linkshell == linkShells[1].name then
				windower.chat.input("/l [" .. packet.payload.from .. "] " .. packet.payload.message:gsub("[\n\r]", " "));
			elseif packet.payload.linkshell == linkShells[2].name then
				windower.chat.input("/l2 [" .. packet.payload.from .. "] " .. packet.payload.message:gsub("[\n\r]", " "));
			end;
            --some stuff to make sure im not sending the message the server told me to send back to the server again
			table.insert(echos, "[" .. packet.payload.from .. "] " .. packet.payload.message:gsub("[\n\r]", " "));
			if #echos > 10 then
				table.remove(echos, 1);
			end;
		else
			removePacketFromBuffers(packet.packetId);
		end;
	end;
end;
--array.includes
function has_value(tab, val)
	for index, value in ipairs(tab) do
		if value == val:gsub("[\n\r]", " ") then
			return true;
		end;
	end;
	return false;
end;
--listen wrapper
function listen()
	handleResponse();
	setupReceiver();
end;
--sends a heartbeat every second
function heartbeat()
	if heartbeatReply then
		heartbeatReply = false;
		sendToBot(buildPacket("HEARTBEAT", {}));
		coroutine.schedule(heartbeat, 1);
	else
		notifiyConnectionLost();
	end;
end;
--make sure the user is the Linkshell holder before sending the add request for the linkshell to the server
function sendLinkshellAddRequest(requestId)
	if framework.playerIsLsOwner() then
        playerData = framework.getPlayerData()
		sendNotice("Sending request to add " .. playerData.linkshell .. " to LinkCloud...");
		sendToBot(buildPacket("ADD_LINKSHELL", {
			linkId = requestId,
			lsName = playerData.linkshell
		}));
	else
		sendError("You must be the Linkshell owner to use this command.  Please ensure you only have YOUR linkshell equipped in the #1 slot.");
	end;
end;

function onLSMesUpdate (lsIndex, text)
    lsData = getLSFromLsmes(text)
    if lsData.shellNumber ~= 0 then
		linkShells[lsIndex] = lsData
        sendToBot(buildPacket("LINKSHELL_UPDATE", {
            linkshellname = linkShells[lsIndex].name
        }));
        sendNotice("Linkshell #" .. lsIndex .. " changed to " .. linkShells[lsIndex].name)
    end
end
function dump(o)
    if type(o) == 'table' then
        local s = '{ '
        for k, v in pairs(o) do
            if type(k) ~= 'number' then
                k = '\"' .. k .. '\"'
            end
            s = s .. '[' .. k .. '] = ' .. dump(v) .. ', '
        end
        return s .. '} '
    else
        return tostring(o)
    end
end

function onLSMessageReceived (lsIndex, text)

    if linkShells[lsIndex] and linkShells[lsIndex].name ~= nil then
        payload = buildMessagePayload(text, linkShells[lsIndex].name);
		
        if not has_value(echos, payload.message) then
		
            table.insert(lsBuffers[lsIndex], buildPacket("LINKSHELL_MESSAGE", payload));
        end;
    end
end

function onShoutReceived (text)
    if framework.settings.ProcessShouts then
        table.insert(otherbuffer, buildPacket("SHOUT", buildShoutPayload(text)));
    end
end

function cmdToggleAutoConnect () 
    framework.settings.AutoConnect = not framework.settings.AutoConnect;
    sendNotice("Auto Connect on Startup: " .. tostring(framework.settings.AutoConnect));
    framework.saveAddonConfig();
end

function cmdToggleAutoReconnect ()
    framework.settings.AutoReconnect = not framework.settings.AutoReconnect;
    sendNotice("Auto Reconnect: " .. tostring(framework.settings.AutoReconnect));
    framework.saveAddonConfig();
end

function cmdToggleShouts ()
    framework.settings.ProcessShouts = not framework.settings.ProcessShouts;
    sendNotice("Sending Shouts: " .. tostring(framework.settings.ProcessShouts));
    framework.saveAddonConfig();
end

function cmdShowVersion (engineVer)
    sendNotice('Engine Version: ' .. engineVer .. ' | LinkCloud Version: ' .. _version)
end
function sendNotice (msg) 
    if(framework.sendNoticeMessage ~= nil) then framework.sendNoticeMessage(msg) end 
    if(framework.consoleLog ~= nil) then framework.consoleLog(msg) end
end
function sendError (msg)
    if(framework.sendErrorMessage ~= nil) then framework.sendErrorMessage(msg) end
    if(framework.consoleLog ~= nil) then framework.consoleLog('ERROR: ' .. msg) end
end
function cmdShowHelpMenu() 
    sendNotice(" *** LinkCloud v" .. _version .. " - Author: Twisted ***");
    sendNotice(" connect | c --> Attempts to connect to the LinkCloud service.");
    sendNotice(" autoconnect --> Toggles Automatic connection when game starts. [Default On]");
    sendNotice(" autoreconnect --> Toggles Automatic reconnection when connection is lost. [Default On]");
    sendNotice(" shouts --> Toggles sending shouts to LinkCloud. [Default On]")
    sendNotice(" version | ver | v --> Displays Version");
    sendNotice(" status | s --> Shows connection status to the server");
    sendNotice(" help | h --> Displays this message");
end


return {
	hook = hook,
	onLSMesUpdate = onLSMesUpdate,
	onLSMessageReceived = onLSMessageReceived,
	onShoutReceived = onShoutReceived,
	sendLinkshellAddRequest = sendLinkshellAddRequest
}